clear
clc

% Data from Kirchner and Weil, (2000) Delayed Biological recovery from
% extinctions throughout the fossil record. Nature, 404, 177-180

% Fits the model of the form presented in Table 8.3
% Uses a saved model saved of the natural form, found by searching for minimum DEV over kappa

disp('Release version of script ExtOrigBookModelFit')

% model using common mid-point time vector

load lgeps.txt  % irregular time series of logarithms of extinctions of genera
load lgops.txt  % irregular time series of logarithms of originations of genera

n1=length(lgeps(:,2));
n2=length(lgops(:,2));

% Use slightly shorter subseries as in published source

sub1=[1:(n1-3)];
sub2=[1:(n2-3)];

lgeps=lgeps(sub1,:);
lgops=lgops(sub2,:);

lgeps(:,1)=545-lgeps(:,1);  % reverse from time before present 
lgops(:,1)=545-lgops(:,1);

mntim=(lgeps(:,1)+lgops(:,1))/2;
% 
lgeps(:,1)=mntim;  % change to a common  mid point time vector
lgops(:,1)=mntim;


n1=length(lgeps(:,2))
n2=length(lgops(:,2))
N=n1+n2;

Initpdev=N*exp((n1*log(var(lgeps(:,2),1)) + n2*log(var(lgops(:,2),1)))/N) % Initial psuedo-deviance
InitAIC=N*log(Initpdev) %  AIC for null model 

% mean correct series for improved conditioning, tho' constant term and trend are estimated
mn(1) = mean(lgeps(:,2));
mn(2) = mean(lgops(:,2));

lgeps(:,2) = lgeps(:,2) - mn(1);
lgops(:,2) = lgops(:,2) - mn(2);

con1=ones(n1,1);
trend1=lgeps(:,1);
con2=ones(n2,1);
trend2=lgops(:,1);

xreg1=[con1,trend1];
xreg2=[con2,trend2];
xreg={xreg1,xreg2};

y={lgeps,lgops};
m=2

maxiter=40
convcrit=0.00001
convcrit=0.000000001
monind=1
controlpar=[maxiter,convcrit,monind];
strategy=[];

load ExtOrigKappasModels % produced in the analysis for web presentation
% these were generated for models of the natural form, searching for minimum DEV over kappa
results=Kmodel{5};

kappa=0.125
p=2
parind=2;
obsnoise=1;
struc=1 

VCZARcoeff=results{1}
VCZARSig=results{2}
Strucoeff=results{6}

% derive predictive model initial values from saved natural models
Phi=VCZARcoeff
SigPhi=(Strucoeff\VCZARSig)/Strucoeff';
[Xi, SigXi, M] = VCZARnatural2predictive(Phi,SigPhi,m,p,kappa)
VCZARcoeff=Xi
VCZARSig=SigXi

NoiseSig=results{3};
xregcoeff=results{4};
Strucoeff=[];
constraints={[],[],[],[0,0]}; % constrained for structural model with no free parameter
% un-comment the next line to allow estimation of the structural noise parameter
% constraints={[],[],[],[0,1]}; % to check structural model parameter is not significant


% uncomment for constrained parameter model as commented on p 263 in book
% VCZARcoeff(1,1,1)=0;
% VCZARcoeff(1,2,2)=0;
% xicon=[0,1,1,0,1,1,1,1]  % March 2014 as commented on p 263 in book
% constraints={xicon,[],[],[0,0]};


initpar={VCZARcoeff,VCZARSig,NoiseSig,Strucoeff};


results=...
    VCZARestimate_reml(y,p,kappa,parind,obsnoise,xreg,struc,initpar,constraints,controlpar,strategy);

model=results;

% save model with structure as in Table 8.3
% save ExtOrigBookModelWeb  model mn kappa m p controlpar parind obsnoise struc constraints

% save model with two non-significant coefficients removed (constrained to zero) as described on p 263
% save ExtOrigBookModelWebC  model mn kappa m p controlpar parind obsnoise struc constraints

warning off MATLAB:divideByZero
format long
VCZARcoeff=results{1}
VCZARSig=results{2}
if struc==0
    VCZARcorr=corrmat(VCZARSig)
end
NoiseSig=results{3}
if obsnoise==2
    Noisecorr=corrmat(NoiseSig)
end

xregcoeff=results{4}
Strucoeff=results{6}
parstderr=results{7};
parcorrmx=results{8};
dev=results{9}
pdev=results{10}
FailInd=results{18}
totmodpar=results{15};
subindex=results{16};
xregtot=results{17};

% presentation of t values of standard parameters
longparstderr=zeros(1,totmodpar);
longparstderr(subindex)=parstderr;
[sVCZARcoeff,sVCZARSig,sNoiseSig,sStrucoeff]=parvec2true_reml(longparstderr,m,p,obsnoise,struc);
tVCZARcoeff=VCZARcoeff./sVCZARcoeff
tVCZARSig=VCZARSig./sVCZARSig
if obsnoise>0
    tNoiseSig=NoiseSig./sNoiseSig
end
if xregtot>0
    xregvarmx=results{5};
    txregcoeff=xregcoeff./sqrt(diag(xregvarmx))
end
if struc>0
    tStrucoeff=Strucoeff./sStrucoeff
end


if struc>0
    VCZARSigNS=(Strucoeff\VCZARSig)/(Strucoeff'); % non-structural form
else
    VCZARSigNS=VCZARSig;
end
if struc==2
    for k=1:p
        VCZARcoeffNS(:,:,k)=Strucoeff\VCZARcoeff(:,:,k);  %standard form
    end
else
    VCZARcoeffNS=VCZARcoeff;
end

format short
