clear
clc

% Properties of model of data from Kirchner and Weil, (2000) Delayed Biological 
% recovery from extinctions throughout the fossil record. Nature, 404, 177-180

disp('Release version of script ExtOrigBookModelProperties')

% specification of frequency and lag ranges for properties
flim=0.15
fnum=500
laglim=100
lagnum=1000
feedback=0  % (1) for closed loop and (0) for open loop responses

% Parameters for simulated delta method of obtaining approximate standard errors
scal=100.0
lim=2; % multiplier of SEs shown in limits for time domain properties
slim=1 % multiplier of SEs shown in limits for spectral properties
shosim=1  % set to 1 for SEs, zero just for estimates.
rep=400 % replications for simulation
% rep=100

m=2


% MODEL IN BOOK BUT !! Fig 8.12 (f) shows SRF with feedback instead of IRF with feedback !!
% ********************************************************************************
load ExtOrigBookModelWeb
kappa=0.125
constraints={[],[],[],[0,0]};

% load ExtOrigBookModelWebC % model with 2 parameters constrained to zero

results=model;
% acquisition and presentation of model coefficients
warning off MATLAB:divideByZero
format long
VCZARcoeff=results{1}
VCZARSig=results{2}
if struc==0
    VCZARcorr=corrmat(VCZARSig)
end
NoiseSig=results{3}
if obsnoise==2
    Noisecorr=corrmat(NoiseSig)
end
xregcoeff=results{4}
Strucoeff=results{6}
parstderr=results{7};
parcorrmx=results{8};
dev=results{9}
pdev=results{10}
FailInd=results{18}
totmodpar=results{15};
subindex=results{16};
xregtot=results{17};

stanperr=results{12};

% presentation of t values of standard parameters
longparstderr=zeros(1,totmodpar);
longparstderr(subindex)=parstderr;
[sVCZARcoeff,sVCZARSig,sNoiseSig,sStrucoeff]=parvec2true_reml(longparstderr,m,p,obsnoise,struc);
tVCZARcoeff=VCZARcoeff./sVCZARcoeff
tVCZARSig=VCZARSig./sVCZARSig
if obsnoise>0
    tNoiseSig=NoiseSig./sNoiseSig
end
if xregtot>0
    xregvarmx=results{5};
    txregcoeff=xregcoeff./sqrt(diag(xregvarmx))
end
if struc>0
    tStrucoeff=Strucoeff./sStrucoeff
end

format short

% transform model coefficients from structural to standard form

if struc>0
    VCZARSigNS=(Strucoeff\VCZARSig)/(Strucoeff'); % non-structural form
else
    VCZARSigNS=VCZARSig;
end
if struc==2
    for k=1:p
        VCZARcoeffNS(:,:,k)=Strucoeff\VCZARcoeff(:,:,k);  %standard form
    end
else
    VCZARcoeffNS=VCZARcoeff;
end

% transform to natural form of model to determine properties
if parind==1
    Phi=VCZARcoeffNS;
    SigPhi=VCZARSigNS;
else
    [Phi, SigPhi, M] = VCZARpredictive2natural(VCZARcoeffNS, VCZARSigNS, m, p, kappa);
end

% eps=1.e-10;
% SigPhi=SigPhi+eps*eye(m); % conditioning in case of singular noise free disturbance variance

[mspec,fvec,wspec,gvec]=VCZARSpecf(Phi,SigPhi,kappa,flim,fnum); % equally spaced un-warped frequencies
nfmspec=mspec; % noise free spectrum


[mcov,lvec]=VCZARCov(Phi,SigPhi,kappa,laglim,lagnum);
nvec=length(lvec);

llvec=[-fliplr(lvec(2:length(lvec))),lvec]; % for negative and positive lag vector

% time domain and spectral properties extracted

nmspec(1,1,:)=mspec(1,1,:);
nmspec(2,2,:)=mspec(2,2,:);
nmspec(2,1,:)=mspec(2,1,:);
nmspec(1,2,:)=mspec(2,1,:);
if obsnoise==1
    nmspec(1,1,:)=mspec(1,1,:)+NoiseSig(1);
    nmspec(2,2,:)=mspec(2,2,:)+NoiseSig(2);
    mcov(1,1,1)=mcov(1,1,1)+NoiseSig(1);
    mcov(2,2,1)=mcov(2,2,1)+NoiseSig(2);
end
if obsnoise==2
    nmspec(1,1,:)=mspec(1,1,:)+NoiseSig(1,1);
    nmspec(2,2,:)=mspec(2,2,:)+NoiseSig(2,2);
    nmspec(2,1,:)=mspec(2,1,:)+NoiseSig(2,1);
    nmspec(1,2,:)=mspec(2,1,:)+NoiseSig(1,2);
    mcov(1,1,1)=mcov(1,1,1)+NoiseSig(1,1);
    mcov(2,2,1)=mcov(2,2,1)+NoiseSig(2,2);
    mcov(2,1,1)=mcov(2,1,1)+NoiseSig(2,1);
    mcov(1,2,1)=mcov(1,2,1)+NoiseSig(1,2);
end

acf_extinct=squeeze(mcov(1,1,:))/mcov(1,1,1);
acf_origin=squeeze(mcov(2,2,:))/mcov(2,2,1);
lccov=squeeze(mcov(1,2,:));
rccov=squeeze(mcov(2,1,:));
lrccov=[flipud(lccov(2:nvec));rccov];
ccf_ext_orig=lrccov/sqrt(mcov(1,1,1)*mcov(2,2,1));

spec_extinct=squeeze(real(mspec(1,1,:)));
spec_origin=squeeze(real(mspec(2,2,:)));
wspec_extinct=squeeze(real(wspec(1,1,:)));
wspec_origin=squeeze(real(wspec(2,2,:)));
coh_ext_orig=abs(squeeze(nmspec(1,2,:))).^2./(real(squeeze(nmspec(1,1,:))).*real(squeeze(nmspec(2,2,:))));
phas_ext_orig=unwrap(angle(squeeze(nmspec(2,1,:))))/(2*pi);

[irf_ext_orig,srf_ext_orig,lvec]=VCZARIrf(Phi,SigPhi,kappa,laglim,lagnum,feedback);

SDacf_extinct=acf_extinct*0.0;
SDacf_origin=acf_origin*0.0;
SDccf_ext_orig=ccf_ext_orig*0.0;
SDlspec_extinct=spec_extinct*0.0;
SDlspec_origin=spec_origin*0.0;
SDwspec_extinct=wspec_extinct*0.0;
SDwspec_origin=wspec_origin*0.0;
SDcoh_ext_orig=coh_ext_orig*0.0;
SDphas_ext_orig=phas_ext_orig*0.0;
SDirf_ext_orig=irf_ext_orig*0.0;
SDsrf_ext_orig=srf_ext_orig*0.0;



% Simulate confidence intervals ***********************************

% parameter perturbations simulated from their covariance matrix
% and properties of perturbed parameters determined

chCM=chol(parcorrmx)';
K=length(parstderr);

if shosim==1
    for ct=1:rep
        ranvec=randn(K,1);
        parpert=parstderr.*(chCM*ranvec)/scal;
        longparpert=zeros(1,totmodpar);
        longparpert(subindex)=parpert;
        [pVCZARcoeff,pVCZARSig,pNoiseSig,pStrucoeff]=parvec2true_reml(longparpert,m,p,obsnoise,struc);
        if struc>0
            pStrucoeff=pStrucoeff-eye(2);
        end
        pVCZARcoeff=pVCZARcoeff+VCZARcoeff;
        pVCZARSig=pVCZARSig+VCZARSig;
        if obsnoise>0
            pNoiseSig=pNoiseSig+NoiseSig;
        end
        if struc>0
            pStrucoeff=pStrucoeff+Strucoeff;
        end
        
        if struc>0
            pVCZARSigNS=(pStrucoeff\pVCZARSig)/(pStrucoeff'); % non-structural form
        else
            pVCZARSigNS=pVCZARSig;
        end
        if struc==2
            for k=1:p
                pVCZARcoeffNS(:,:,k)=pStrucoeff\pVCZARcoeff(:,:,k);  %standard form
            end
        else
            pVCZARcoeffNS=pVCZARcoeff;
        end
        
        
        if parind==1
            pPhi=pVCZARcoeffNS;
            pSigPhi=pVCZARSigNS;
        else
            [pPhi, pSigPhi, M] = VCZARpredictive2natural(pVCZARcoeffNS, pVCZARSigNS, m, p, kappa);
        end
        
        [pmspec,fvec,pwspec,gvec]=VCZARSpecf(pPhi,pSigPhi,kappa,flim,fnum);
        pnfmspec=pmspec; % noise free spectrum
        
        [pmcov,lvec]=VCZARCov(pPhi,pSigPhi,kappa,laglim,lagnum);
        
        pnmspec(1,1,:)=pmspec(1,1,:);
        pnmspec(2,2,:)=pmspec(2,2,:);
        pnmspec(2,1,:)=pmspec(2,1,:);
        pnmspec(1,2,:)=pmspec(2,1,:);
        
        if obsnoise==1
            pnmspec(1,1,:)=pmspec(1,1,:)+pNoiseSig(1);
            pnmspec(2,2,:)=pmspec(2,2,:)+pNoiseSig(2);
            pmcov(1,1,1)=pmcov(1,1,1)+pNoiseSig(1);
            pmcov(2,2,1)=pmcov(2,2,1)+pNoiseSig(2);
        end
        
        
        if obsnoise==2
            pnmspec(1,1,:)=pmspec(1,1,:)+pNoiseSig(1,1);
            pnmspec(2,2,:)=pmspec(2,2,:)+pNoiseSig(2,2);
            pnmspec(2,1,:)=pmspec(2,1,:)+pNoiseSig(2,1);
            pnmspec(1,2,:)=pmspec(2,1,:)+pNoiseSig(1,2);
            pmcov(1,1,1)=pmcov(1,1,1)+pNoiseSig(1,1);
            pmcov(2,2,1)=pmcov(2,2,1)+pNoiseSig(2,2);
            pmcov(2,1,1)=pmcov(2,1,1)+pNoiseSig(2,1);
            pmcov(1,2,1)=pmcov(1,2,1)+pNoiseSig(1,2);
        end        
        pspec_extinct=squeeze(real(pmspec(1,1,:)));
        pspec_origin=squeeze(real(pmspec(2,2,:)));
        pwspec_extinct=squeeze(real(pwspec(1,1,:)));
        pwspec_origin=squeeze(real(pwspec(2,2,:)));
        
        pacf_extinct=squeeze(pmcov(1,1,:))/pmcov(1,1,1);
        pacf_origin=squeeze(pmcov(2,2,:))/pmcov(2,2,1);
        plccov=squeeze(pmcov(1,2,:));
        prccov=squeeze(pmcov(2,1,:));
        plrccov=[flipud(plccov(2:nvec));prccov];
        pccf_ext_orig=plrccov/sqrt(pmcov(1,1,1)*pmcov(2,2,1));
        
        pspec_extinct=squeeze(real(pmspec(1,1,:)));
        pspec_origin=squeeze(real(pmspec(2,2,:)));
        pwspec_extinct=squeeze(real(pwspec(1,1,:)));
        pwspec_origin=squeeze(real(pwspec(2,2,:)));
        pcoh_ext_orig=abs(squeeze(pnmspec(1,2,:))).^2./(real(squeeze(pnmspec(1,1,:))).*real(squeeze(pnmspec(2,2,:))));
        pphas_ext_orig=unwrap(angle(squeeze(pnmspec(2,1,:))))/(2*pi);
        
        [pirf_ext_orig,psrf_ext_orig,lvec]=VCZARIrf(pPhi,pSigPhi,kappa,laglim,lagnum,feedback);
        
        % perturbations completed and used to accumulate standard deviations
        
        SDacf_extinct=SDacf_extinct+(atanh(pacf_extinct)-atanh(acf_extinct)).^2;
        SDacf_origin=SDacf_origin+(atanh(pacf_origin)-atanh(acf_origin)).^2;
        SDccf_ext_orig=SDccf_ext_orig+(atanh(pccf_ext_orig)-atanh(ccf_ext_orig)).^2;
        SDlspec_extinct=SDlspec_extinct+(log(pspec_extinct)-log(spec_extinct)).^2;
        SDlspec_origin=SDlspec_origin+(log(pspec_origin)-log(spec_origin)).^2;
        SDwspec_extinct=SDwspec_extinct+(log(pwspec_extinct)-log(wspec_extinct)).^2;
        SDwspec_origin=SDwspec_origin+(log(pwspec_origin)-log(wspec_origin)).^2;
        SDcoh_ext_orig=SDcoh_ext_orig+(atanh(sqrt(pcoh_ext_orig))-atanh(sqrt(coh_ext_orig))).^2;
        SDphas_ext_orig=SDphas_ext_orig+(pphas_ext_orig-phas_ext_orig).^2;
        SDirf_ext_orig=SDirf_ext_orig+(pirf_ext_orig-irf_ext_orig).^2;
        SDsrf_ext_orig=SDsrf_ext_orig+(psrf_ext_orig-srf_ext_orig).^2;
        
    end
end

% re-scaling of standard deviations

SDacf_extinct=sqrt(SDacf_extinct/rep)*scal;
SDacf_origin=sqrt(SDacf_origin/rep)*scal;
SDccf_ext_orig=sqrt(SDccf_ext_orig/rep)*scal;
SDlspec_extinct=sqrt(SDlspec_extinct/rep)*scal;
SDlspec_origin=sqrt(SDlspec_origin/rep)*scal;
SDwspec_extinct=sqrt(SDwspec_extinct/rep)*scal;
SDwspec_origin=sqrt(SDwspec_origin/rep)*scal;
SDcoh_ext_orig=sqrt(SDcoh_ext_orig/rep)*scal;
SDphas_ext_orig=sqrt(SDphas_ext_orig/rep)*scal;
SDirf_ext_orig=sqrt(SDirf_ext_orig/rep)*scal;
SDsrf_ext_orig=sqrt(SDsrf_ext_orig/rep)*scal;

% plotting of model properties

disp('Spectrum extinctions integral')
specint=(sum(spec_extinct)-0.5*spec_extinct(1))*flim/fnum

disp('Spectrum originations integral')
specint=(sum(spec_origin)-0.5*spec_origin(1))*flim/fnum 

figure(11)
plot(fvec,spec_extinct,'-k','linewidth', 2)
hold on
if shosim==1
    plot(fvec,spec_extinct.*exp(SDlspec_extinct*slim),'--k','linewidth', 2)
    plot(fvec,spec_extinct./exp(SDlspec_extinct*slim),'--k','linewidth', 2)
end
plot(fvec([1,length(fvec)]),[0,0],'-k','linewidth', 2)
hold off
set(gca,'fontsize',12)
set(gca,'PlotBoxAspectRatio',[1.5,1,1])
ylabel('Spectrum of extinctions','Fontsize',20)
xlabel('Frequency in cycles/Myr','Fontsize',20)
axis([0,0.1,0,7.5])

figure(12)
plot(fvec,spec_origin,'-k','linewidth', 2)
hold on
if shosim==1
    plot(fvec,spec_origin.*exp(SDlspec_origin*slim),'--k','linewidth', 2)
    plot(fvec,spec_origin./exp(SDlspec_origin*slim),'--k','linewidth', 2)
end
plot(fvec([1,length(fvec)]),[0,0],'-k','linewidth', 2)
hold off
set(gca,'fontsize',12)
set(gca,'PlotBoxAspectRatio',[1.5,1,1])
ylabel('Spectrum of originations','Fontsize',20)
xlabel('Frequency in cycles/Myr','Fontsize',20)
axis([0,0.1,0,13])

figure(13)
plot(gvec,wspec_extinct,'-k','linewidth', 2)
hold on
if shosim==1
    plot(gvec,wspec_extinct.*exp(SDwspec_extinct*slim),'--k','linewidth', 2)
    plot(gvec,wspec_extinct./exp(SDwspec_extinct*slim),'--k','linewidth', 2)
end
plot(gvec([1,length(gvec)]),[0,0],'-k','linewidth', 2)
hold off
set(gca,'fontsize',12)
set(gca,'PlotBoxAspectRatio',[1.5,1,1])
ylabel('Warped spectrum of extinctions','Fontsize',20)
xlabel('Frequency in cycles/Myr','Fontsize',20)
axis([0,0.5,0,0.55])

figure(14)
plot(gvec,wspec_origin,'-k','linewidth', 2)
hold on
if shosim==1
    plot(gvec,wspec_origin.*exp(SDwspec_origin*slim),'--k','linewidth', 2)
    plot(gvec,wspec_origin./exp(SDwspec_origin*slim),'--k','linewidth', 2)
end
plot(gvec([1,length(gvec)]),[0,0],'-k','linewidth', 2)
hold off
set(gca,'fontsize',12)
set(gca,'PlotBoxAspectRatio',[1.5,1,1])
ylabel('Warped spectrum of originations','Fontsize',20)
xlabel('Frequency in cycles/Myr','Fontsize',20)
axis([0,0.5,0,0.9])


figure(15)
plot(lvec,acf_extinct,'-k','linewidth', 2)
hold on
if shosim==1
    plot(lvec,tanh(atanh(acf_extinct)+SDacf_extinct*lim),'--k','linewidth', 2)
    plot(lvec,tanh(atanh(acf_extinct)-SDacf_extinct*lim),'--k','linewidth', 2)
end
plot(lvec([1,length(lvec)]),[0,0],'-k','linewidth', 2)
hold off
set(gca,'fontsize',12)
set(gca,'PlotBoxAspectRatio',[1.5,1,1])
ylabel('ACF of extinctions','Fontsize',20)
xlabel('Time lag in Myr','Fontsize',20)
axis([0,laglim,-0.2,1.0])




figure(16)
plot(lvec,acf_origin,'-k','linewidth', 2)
hold on
if shosim==1
    plot(lvec,tanh(atanh(acf_origin)+SDacf_origin*lim),'--k','linewidth', 2)
    plot(lvec,tanh(atanh(acf_origin)-SDacf_origin*lim),'--k','linewidth', 2)
end
plot(lvec([1,length(lvec)]),[0,0],'-k','linewidth', 2)
hold off
set(gca,'fontsize',12)
set(gca,'PlotBoxAspectRatio',[1.5,1,1])
ylabel('ACF of originations','Fontsize',20)
xlabel('Time lag in Myr','Fontsize',20)
axis([0,laglim,-0.3,1.0])



figure(17)
plot(llvec,ccf_ext_orig,'-k','linewidth', 2)
hold on
if shosim==1
    plot(llvec,tanh(atanh(ccf_ext_orig)+SDccf_ext_orig*lim),'--k','linewidth', 2)
    plot(llvec,tanh(atanh(ccf_ext_orig)-SDccf_ext_orig*lim),'--k','linewidth', 2)
end
plot(llvec([1,length(llvec)]),[0,0],'-k','linewidth', 2)
plot(llvec([1,length(llvec)]),[0,0],'-k','linewidth', 2)
plot([0,0],[-1,1],'-k','linewidth', 1)
hold off
set(gca,'fontsize',12)
set(gca,'PlotBoxAspectRatio',[3,1,1])
title('CCF of originations with lagged extinctions','Fontsize',14)
xlabel('Time lag in Myr','Fontsize',14)
axis([-laglim,laglim,-0.3,0.6])


figure(18)
plot(fvec,coh_ext_orig,'-k','linewidth', 2)
hold on
if shosim==1
    plot(fvec,tanh(atanh(sqrt(coh_ext_orig))+SDcoh_ext_orig*lim).^2,'--k','linewidth', 2)
    cohlow=tanh(atanh(sqrt(coh_ext_orig))-SDcoh_ext_orig*lim);
    cohlow=(cohlow.*(cohlow>0)).^2;
    plot(fvec,cohlow,'--k','linewidth', 2)
end
plot(fvec([1,length(fvec)]),[0,0],'-k','linewidth', 2)
hold off
set(gca,'PlotBoxAspectRatio',[1.5,1,1])
set(gca,'fontsize',12)
title('Coherency between originations and extinctions','Fontsize',20)
xlabel('Frequency in cycles/Myr','Fontsize',20)
axis([0,flim,0,1])

figure(19)
plot(fvec,phas_ext_orig,'-k','linewidth', 2)
hold on
if shosim==1
    plot(fvec,phas_ext_orig+SDphas_ext_orig*lim,'--k','linewidth', 2)
    plot(fvec,phas_ext_orig-SDphas_ext_orig*lim,'--k','linewidth', 2)
end
plot(fvec([1,length(fvec)]),[0,0],'-k','linewidth', 2)
hold off
set(gca,'fontsize',12)
set(gca,'PlotBoxAspectRatio',[1.5,1,1])
title('Phase from extinctions to originations','Fontsize',20)
xlabel('Frequency in cycles/Myr','Fontsize',20)
axis([0,flim,0,0.8])

figure(20) % used for both impulse and step response for feedback = 1 or 0
plot(lvec,irf_ext_orig,'-k','linewidth', 2)
hold on
if shosim==1
    plot(lvec,irf_ext_orig+SDirf_ext_orig*lim,'--k','linewidth', 2)
    plot(lvec,irf_ext_orig-SDirf_ext_orig*lim,'--k','linewidth', 2)
end
plot(lvec([1,length(lvec)]),[0,0],'-k','linewidth', 2)
hold off
set(gca,'PlotBoxAspectRatio',[1.5,1,1])
set(gca,'fontsize',12)
xlabel('Response time in Myr','Fontsize',20)
title('Impulse response from extinctions to originations','Fontsize',20)
axis([0,laglim,-0.4,0.4])

figure(21) % used for both impulse and step response for feedback = 1 or 0
plot(lvec,srf_ext_orig,'-k','linewidth', 2)
hold on
if shosim==1
    plot(lvec,srf_ext_orig+SDsrf_ext_orig*lim,'--k','linewidth', 2)
    plot(lvec,srf_ext_orig-SDsrf_ext_orig*lim,'--k','linewidth', 2)
end
plot(lvec([1,length(lvec)]),[0,0],'-k','linewidth', 2)
hold off
set(gca,'PlotBoxAspectRatio',[1.5,1,1])
set(gca,'fontsize',12)
xlabel('Response time in Myr','Fontsize',20)
title('Step response from extinctions to originations','Fontsize',20)
axis([0,laglim,-1.8,2.5])

