clear
clc
% Script to identify the Kappa coefficient of a CZAR model for the Extinction - Originations series
% Uses model order 2 selected by considerations of AIC using kappa = 0.3 recommended in the book
% Models for a range of kappa values are saved in a .mat file
% natural parameterisation with structural disturbance noise model give best fits

disp('Release version of script ExtOrigKappasDEV')

% Data from Kirchner and Weil, (2000) Delayed Biological recovery from
% extinctions throughout the fossil record. Nature, 404, 177-180

% model using common mid-point time vector

load lgeps.txt  % irregular time series of logarithms of extinctions of genera
load lgops.txt  % irregular time series of logarithms of originations of genera

n1=length(lgeps(:,2));
n2=length(lgops(:,2));

% Use slightly shorter subseries as in published source

sub1=[1:(n1-3)];
sub2=[1:(n2-3)];

lgeps=lgeps(sub1,:);
lgops=lgops(sub2,:);

lgeps(:,1)=545-lgeps(:,1);  % reverse from time before present 
lgops(:,1)=545-lgops(:,1);

mntim=(lgeps(:,1)+lgops(:,1))/2;
% 
lgeps(:,1)=mntim;  % change to a common  mid point time vector
lgops(:,1)=mntim;

n1=length(lgeps(:,2))
n2=length(lgops(:,2))
N=n1+n2;

Initpdev=N*exp((n1*log(var(lgeps(:,2),1)) + n2*log(var(lgops(:,2),1)))/N) % Initial psuedo-deviance
InitAIC=N*log(Initpdev) %  AIC for null model 

% mean correct series for improved conditioning, tho' constant term and trend are estimated
mn(1) = mean(lgeps(:,2));
mn(2) = mean(lgops(:,2));

lgeps(:,2) = lgeps(:,2) - mn(1);
lgops(:,2) = lgops(:,2) - mn(2);



y={lgeps,lgops};
m=2


con1=ones(n1,1);
trend1=lgeps(:,1);
con2=ones(n2,1);
trend2=lgops(:,1);
xreg1=[con1,trend1];
xreg2=[con2,trend2];
xreg={xreg1,xreg2};




initpar={}
maxiter=40
convcrit=0.00001
monind=1
controlpar=[maxiter,convcrit,monind];


strategy=[];



kappavec=0.05*[1.0,1.25,1.6,2.0,2.5,3.15,4.0,5.0,6.3,8.0,10.0]

p=2
parind=1;
obsnoise=1;
struc=1
constraints={}; % default for structural model remove next line for this option
constraints={[],[],[],[0,1]}; % structural model series dependence reversed from default

nkappa=length(kappavec);
DEVmat=zeros(1,nkappa);
PDEVmat=DEVmat;

for k=1:nkappa
    
    kappa=kappavec(k)
    
    load ExtOrigOrdersModels model
    Iresults=model{2}; % used saved model order 2 for Initial values
      
    IVCZARcoeff=Iresults{1};
    IVCZARSig=Iresults{2};
    INoiseSig=Iresults{3};
    IStrucoeff=Iresults{5}; 

    initpar={IVCZARcoeff,IVCZARSig,INoiseSig,IStrucoeff}; % for reml estimation
    
    results=...
        VCZARestimate_reml(y,p,kappa,parind,obsnoise,xreg,struc,initpar,constraints,controlpar,strategy);
    
    Failind=results{18} % to check on satisfactory convergence
    
    Kmodel{k}=results; % save model for each kappa
    dev=results{9};
    pdev=results{10};
    DEVvec(k)=dev;
    pDEVvec(k)=pdev;
end

% save ExtOrigKappasModels Kmodel m mn xreg nkappa kappavec parind obsnoise struc constraints DEVvec pDEVvec 


figure(10)
plot(kappavec,DEVvec,'-b','linewidth',2)
xlabel('Kappa coefficient','Fontsize', 15)
ylabel('Deviance','Fontsize', 15)