clear
clc
% Script to identify the order of a CZAR model for the Extinction - Originations series
% Uses a kappa value 0.3 set by considerations given in the book
% Models up to order 4 are saved in a .mat file
% natural parameterisation with structural disturbance noise model give best fits
disp('Release version of script ExtOrigOrdersAIC')

% Data from Kirchner and Weil, (2000) Delayed Biological recovery from
% extinctions throughout the fossil record. Nature, 404, 177-180

% model using common mid-point time vector


load lgeps.txt  % irregular time series of logarithms of extinctions of genera
load lgops.txt  % irregular time series of logarithms of originations of genera

n1=length(lgeps(:,2));
n2=length(lgops(:,2));

% Use slightly shorter subseries as in published source

sub1=[1:(n1-3)];
sub2=[1:(n2-3)];

lgeps=lgeps(sub1,:);
lgops=lgops(sub2,:);

lgeps(:,1)=545-lgeps(:,1);  % reverse from time before present 
lgops(:,1)=545-lgops(:,1);

mntim=(lgeps(:,1)+lgops(:,1))/2;
% 
lgeps(:,1)=mntim;  % change to a common  mid point time vector
lgops(:,1)=mntim;

n1=length(lgeps(:,2))
n2=length(lgops(:,2))
N=n1+n2;

Initpdev=N*exp((n1*log(var(lgeps(:,2),1)) + n2*log(var(lgops(:,2),1)))/N) % Initial psuedo-deviance
InitAIC=N*log(Initpdev) %  AIC for null model 

% mean correct series for improved conditioning, tho' constant term and trend are estimated

mn(1) = mean(lgeps(:,2));
mn(2) = mean(lgops(:,2));

lgeps(:,2) = lgeps(:,2) - mn(1);
lgops(:,2) = lgops(:,2) - mn(2);


y={lgeps,lgops}; % time series for model
m=2


con1=ones(n1,1);
trend1=lgeps(:,1);
con2=ones(n2,1);
trend2=lgops(:,1);
xreg1=[con1,trend1];
xreg2=[con2,trend2];
xreg={xreg1,xreg2}; % trend regression specified

maxiter=40
convcrit=0.000001
monind=1
controlpar=[maxiter,convcrit,monind];


strategy=[];


kappa=0.3 % specified in book by consideration of mean interval between observations

parind=1; % natural model parind = 1 is a better conditioned parametrisation than predictive model 
          % with parind = 2 though they are equivalent parameterisations for this exercise

struc=1; % structural noise model struc = 1 is a better conditioned parametrisation than standard
         % with struc = 0 though they are also equivalent parameterisations for this exercise

constraints={}; % default for structural model remove next line for this option
constraints={[],[],[],[],[0,1]}; % structural model series dependence reversed from default
initpar={} % default for simple model
% fit no observation noise model for order 1 
% to provide initial parameters for model with observation noise
p=1
obsnoise=0; 
results=...
    VCZARestimate(y,p,kappa,parind,obsnoise,xreg,struc,initpar,constraints,controlpar,strategy);

% use to set initpar for observation noise model of order 1
obsnoise=1;
VCZARcoeff=results{1};
VCZARSig=results{2};
NoiseSig=0.05*diag(VCZARSig); % specify a small fraction of disturbance noise as initial observation noise
xregcoeff=results{4};
Strucoeff=results{5};
initpar={VCZARcoeff, VCZARSig,NoiseSig,xregcoeff,Strucoeff};


nord=4;
DEVvec=zeros(nord,1);
PDEVvec=DEVvec;

for p=1:nord
    p
    results=... % fit the model
        VCZARestimate(y,p,kappa,parind,obsnoise,xreg,struc,initpar,constraints,controlpar,strategy);
    
    % specify initial values of model order p+1 from parameters of model order p
    if p<nord
        VCZARcoeff=results{1};
        VCZARcoeff(:,:,p+1)=zeros(m); % extend model coefficients
        VCZARSig=results{2};
        NoiseSig=results{3};
        xregcoeff=results{4};
        Strucoeff=results{5};
        initpar={VCZARcoeff,VCZARSig,NoiseSig,xregcoeff,Strucoeff};
    end
    if struc==0
    disp('Correlation matrix of disturbance noise')
        CM=corrmat(results{2})
    else
    disp('Structure of disturbance noise')
        Strucoeff=results{5}
        VCZARSig=results{2}
    end
    Failind=results{17} % to check on satisfactory convergence
    model{p}=results; % save model for each order
    dev=results{8}
    pdev=results{9}
    DEVvec(p)=dev;
    pDEVvec(p)=pdev;
    
end

for p=1:nord
    AICvec(p)=DEVvec(p)+2*m*m*p; 
end



% save ExtOrigOrdersModels model m mn xreg nord kappa parind obsnoise struc constraints DEVvec pDEVvec AICvec

pvec=[1:4];
figure(10)
plot(pvec,AICvec,'-b','linewidth',2)
xlabel('Model order','Fontsize', 15)
ylabel('AIC','Fontsize', 15)
